#! /usr/bin/python

import sys, os
from optparse import OptionParser

import zipfile

container_in = '''<?xml version="1.0"?>
<container version="1.0" xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
  <rootfiles>
    <rootfile full-path="OEBPS/%s" media-type="application/oebps-package+xml"/>
  </rootfiles>
</container>
'''

class EPubFile:
    def __init__(self, name):
        self.epub = zipfile.ZipFile(outname, 'w')
        self.addData('application/epub+zip', '', 'mimetype')

    def __del__(self):
        self.close()

    def close(self):
        if self.epub is not None:
            self.epub.close()
            self.epub = None

    def addFile(self, fullname, subdir, relativeName):
        self.epub.write(fullname, os.path.join(subdir, relativeName))

    def addData(self, data, subdir, relativeName):
        self.epub.writestr(os.path.join(subdir, relativeName), data)

class OEBFiles:
    def __init__(self, dirname):
        assert os.path.isdir(dirname)

        if dirname[-1] == os.path.sep:
            self.dirname = dirname[:-1]
        else:
            self.dirname = dirname

        self.names = []
        self._relname = len(self.dirname) + 1

        os.path.walk(self.dirname, self.addDir, None)

    def addDir(self, dummy, dirname, files):
        assert dirname.startswith(self.dirname)

        relname = dirname[self._relname:]

        for file in files:
            self.names.append((os.path.join(dirname, file), os.path.join(relname, file)))

parser = OptionParser(usage = '%prog [options] oebdir [epubfilename]')

options, args = parser.parse_args()

if len(args) < 1 or not os.path.isdir(args[0]):
    parser.print_usage()
    sys.exit(1)

if len(args) == 1:
    outname = args[0]
    if outname[-1] == os.path.sep:
        outname = outname[:-1]

    outname += '.epub'
else:
    outname = args[1]

oebfiles = OEBFiles(args[0])

opfs = [ itsname for dummy, itsname in oebfiles.names if itsname.endswith('.opf') ]

if len(opfs) == 0:
    print >> sys.stderr, 'No .opf files found'
    sys.exit(2)
elif len(opfs) != 1:
    print >> sys.stderr, 'Found too many .opf files:', opfs
    sys.exit(3)

epub = EPubFile(outname)

epub.addData(container_in % opfs[0], 'META-INF', 'container.xml')

for realfile, itsname in oebfiles.names:
    epub.addFile(realfile, 'OEBPS', itsname)

epub.close()
epub = None

# vim:ts=4:sw=4:et
